/* THIS FILE IS AUTOGENERATED - DO NOT CHANGE IT! */

struct hub_config
{
	int   hub_enabled;                     /*<<< Is server enabled (default: 1) */
	int   server_port;                     /*<<< Server port to bind to (default: 1511) */
	char* server_bind_addr;                /*<<< Server bind address (default: "any") */
	int   server_listen_backlog;           /*<<< Server listen backlog (default: 50) */
	char* server_alt_ports;                /*<<< Comma separated list of alternative ports to listen to (default: "") */
	int   show_banner;                     /*<<< Show banner on connect (default: 1) */
	int   show_banner_sys_info;            /*<<< Show banner on connect (default: 1) */
	int   max_users;                       /*<<< Maximum number of users allowed on the hub (default: 500) */
	int   registered_users_only;           /*<<< Allow registered users only (default: 0) */
	int   register_self;                   /*<<< Allow users to register themselves on the hub. (default: 0) */
	int   obsolete_clients;                /*<<< Support obsolete clients using a ADC protocol prior to 1.0 (default: 0) */
	int   chat_is_privileged;              /*<<< Allow chat for operators and above only (default: 0) */
	char* hub_name;                        /*<<< Name of hub (default: "uhub") */
	char* hub_description;                 /*<<< Short hub description, topic or subject. (default: "no description") */
	char* redirect_addr;                   /*<<< A common hub redirect address. (default: "") */
	int   max_recv_buffer;                 /*<<< Max read buffer before parse, per user (default: 4096) */
	int   max_send_buffer;                 /*<<< Max send buffer before disconnect, per user (default: 131072) */
	int   max_send_buffer_soft;            /*<<< Max send buffer before message drops, per user (default: 98304) */
	int   low_bandwidth_mode;              /*<<< Enable bandwidth saving measures (default: 0) */
	int   max_chat_history;                /*<<< Number of chat messages kept in history (default: 20) */
	int   max_logout_log;                  /*<<< Number of log entries for people leaving the hub (default: 20) */
	int   limit_max_hubs_user;             /*<<< Max concurrent hubs as a guest user (default: 10) */
	int   limit_max_hubs_reg;              /*<<< Max concurrent hubs as a registered user (default: 10) */
	int   limit_max_hubs_op;               /*<<< Max concurrent hubs as a operator (or admin) (default: 10) */
	int   limit_max_hubs;                  /*<<< Max total hub connections allowed, user/reg/op combined. (default: 25) */
	int   limit_min_hubs_user;             /*<<< Minimum concurrent hubs as a guest user (default: 0) */
	int   limit_min_hubs_reg;              /*<<< Minimum concurrent hubs as a registered user (default: 0) */
	int   limit_min_hubs_op;               /*<<< Minimum concurrent hubs as a operator (or admin) (default: 0) */
	int   limit_min_share;                 /*<<< Limit minimum share size in megabytes (default: 0) */
	int   limit_max_share;                 /*<<< Limit maximum share size in megabytes (default: 0) */
	int   limit_min_slots;                 /*<<< Limit minimum number of upload slots open per user (default: 0) */
	int   limit_max_slots;                 /*<<< Limit minimum number of upload slots open per user (default: 0) */
	int   flood_ctl_interval;              /*<<< Time interval in seconds for flood control check. (default: 0) */
	int   flood_ctl_chat;                  /*<<< Max chat messages allowed in time interval (default: 0) */
	int   flood_ctl_connect;               /*<<< Max connections requests allowed in time interval (default: 0) */
	int   flood_ctl_search;                /*<<< Max search requests allowed in time interval (default: 0) */
	int   flood_ctl_update;                /*<<< Max updates allowed in time interval (default: 0) */
	int   flood_ctl_extras;                /*<<< Max extra messages allowed in time interval (default: 0) */
	int   tls_enable;                      /*<<< Enable SSL/TLS support (default: 0) */
	int   tls_require;                     /*<<< If SSL/TLS enabled, should it be required (default: 0) */
	char* tls_require_redirect_addr;       /*<<< A redirect address in case a client connects using "adc://" when "adcs://" is required. (default: "") */
	char* tls_certificate;                 /*<<< Certificate file (default: "") */
	char* tls_private_key;                 /*<<< Private key file (default: "") */
	char* file_acl;                        /*<<< File containing access control lists (default: "") */
	char* file_plugins;                    /*<<< Plugin configuration file (default: "") */
	char* msg_hub_full;                    /*<<< "Hub is full" */
	char* msg_hub_disabled;                /*<<< "Hub is disabled" */
	char* msg_hub_registered_users_only;   /*<<< "Hub is for registered users only" */
	char* msg_inf_error_nick_missing;      /*<<< "No nickname given" */
	char* msg_inf_error_nick_multiple;     /*<<< "Multiple nicknames given" */
	char* msg_inf_error_nick_invalid;      /*<<< "Nickname is invalid" */
	char* msg_inf_error_nick_long;         /*<<< "Nickname too long" */
	char* msg_inf_error_nick_short;        /*<<< "Nickname too short" */
	char* msg_inf_error_nick_spaces;       /*<<< "Nickname cannot start with spaces" */
	char* msg_inf_error_nick_bad_chars;    /*<<< "Nickname contains invalid characters" */
	char* msg_inf_error_nick_not_utf8;     /*<<< "Nickname is not valid UTF-8" */
	char* msg_inf_error_nick_taken;        /*<<< "Nickname is already in use" */
	char* msg_inf_error_nick_restricted;   /*<<< "Nickname cannot be used on this hub" */
	char* msg_inf_error_cid_invalid;       /*<<< "CID is not valid" */
	char* msg_inf_error_cid_missing;       /*<<< "CID is not specified" */
	char* msg_inf_error_cid_taken;         /*<<< "CID is taken" */
	char* msg_inf_error_pid_missing;       /*<<< "PID is not specified" */
	char* msg_inf_error_pid_invalid;       /*<<< "PID is invalid" */
	char* msg_ban_permanently;             /*<<< "Banned permanently" */
	char* msg_ban_temporarily;             /*<<< "Banned temporarily" */
	char* msg_auth_invalid_password;       /*<<< "Password is wrong" */
	char* msg_auth_user_not_found;         /*<<< "User not found in password database" */
	char* msg_error_no_memory;             /*<<< "No memory" */
	char* msg_user_share_size_low;         /*<<< "User is not sharing enough" */
	char* msg_user_share_size_high;        /*<<< "User is sharing too much" */
	char* msg_user_slots_low;              /*<<< "User have too few upload slots." */
	char* msg_user_slots_high;             /*<<< "User have too many upload slots." */
	char* msg_user_hub_limit_low;          /*<<< "User is on too few hubs." */
	char* msg_user_hub_limit_high;         /*<<< "User is on too many hubs." */
	char* msg_user_flood_chat;             /*<<< "Chat flood detected, messages are dropped." */
	char* msg_user_flood_connect;          /*<<< "Connect flood detected, connection refused." */
	char* msg_user_flood_search;           /*<<< "Search flood detected, search is stopped." */
	char* msg_user_flood_update;           /*<<< "Update flood detected." */
	char* msg_user_flood_extras;           /*<<< "Flood detected." */
	char* msg_proto_no_common_hash;        /*<<< "No common hash algorithm." */
	char* msg_proto_obsolete_adc0;         /*<<< "Client is using an obsolete ADC protocol version." */
};

